package util;

import java.util.Iterator;

/**
 * Eine geordnete Sammlung (auch bekannt als Folge).
 * Der Benutzer dieser Schnittstelle hat
 * die genaue Kontrolle darueber, wo in der Liste jedes
 * Element eingefuegt wird. Der Benutzer
 * kann auf Elemente anhand ihres ganzzahligen Indexes
 * (Position in der Liste) zugreifen
 * und nach Elementen in der Liste suchen.
 *
 * Im Gegensatz zu Sets erlauben Listen typischerweise
 * doppelte Elemente. Genauer gesagt
 * erlauben Listen normalerweise Paare von Elementen 
 * e1 und e2, sodass e1.equals(e2) und
 * sie erlauben normalerweise mehrere null-Elemente. 
 */
public interface MyList<T> extends Iterable<T>{
  
  /**
   * Fuegt das angegebene Element an das Ende dieser Liste an.
   *
   * @param item das anzuhaengende Element
   */
  public void add(T item);
  
  /**
   * Fuegt das angegebene Element an der angegebenen Position in diese Liste ein
   * (optionale Operation). Verschiebt das derzeit an dieser Position vorhandene
   * Element (sofern vorhanden) und alle folgenden Elemente eins nach rechts.
   *
   * @param index   Index, an dem das angegebene Element eingefuegt werden soll
   * @param item Element, das eingefuegt werden soll
   */
  public void add(int index, T item);
  
  /**
   * Ersetzt das Element an der angegebenen Position in dieser Liste durch das
   * angegebene Element (optionale Operation).
   *
   * @param index   Index des zu ersetzenden Elements
   * @param item Element, das an der angegebenen Position gespeichert werden soll
   * @return das vorher an der angegebenen Position gespeicherte Element
   */
  public void set(int index, T item);
  
  /**
   * Gibt das Element an der angegebenen Position in dieser Liste zurueck.
   *
   * @param index Index des zurueckzugebenden Elements
   * @return das Element an der angegebenen Position in dieser Liste
   */
  public T get(int index);
  
  /**
   * Gibt die Anzahl der Elemente in dieser Liste zurueck. Wenn diese Liste
   * mehr als Integer.MAX_VALUE Elemente enthaelt, wird Integer.MAX_VALUE
   * zurueckgegeben (letztes kann fuer die Aufgabe ignoriert werden).
   *
   * @return die Anzahl der Elemente in dieser Liste
   */
  public int size();
  
  /**
   * Gibt true zurueck, wenn diese Liste keine Elemente enthaelt.
   *
   * @return true, wenn diese Liste keine Elemente enthaelt
   */
  public boolean isEmpty();
  
  /**
   * Entfernt das erste Vorkommen des angegebenen Elements aus dieser Liste,
   * falls es vorhanden ist (optionale Operation). Wenn diese Liste das Element
   * nicht enthaelt, bleibt sie unveraendert.
   *
   * @param o das zu entfernende Element, falls vorhanden
   * @return true, wenn diese Liste das angegebene Element enthielt
   */
  public boolean remove(int index);
  
  /**
   * Gibt true zurueck, wenn diese Liste das angegebene Element enthaelt. Genauer
   * gesagt wird true zurueckgegeben, wenn und nur wenn diese Liste mindestens
   * ein Element e enthaelt, sodass (o==null ? e==null : o.equals(e)).
   *
   * @param o das zu testende Element
   * @return true, wenn diese Liste das angegebene Element enthaelt
   * @throws ClassCastException - wenn der Typ des angegebenen Elements nicht
   *         mit dieser Liste kompatibel ist (optional)
   * @throws NullPointerException - wenn das angegebene Element null ist und
   *         diese Liste keine null-Elemente erlaubt (optional)
   */
  public boolean contains(T item);
  
  /**
   * Gibt den Index der ersten Vorkommens des angegebenen Elements in dieser Liste
   * zurück, oder -1, wenn diese Liste das Element nicht enthält.
   *
   * @param o das zu suchende Element
   * @return den Index der ersten Vorkommens des angegebenen Elements in dieser
   *         Liste, oder -1, wenn diese Liste das Element nicht enthält
   */
  public int indexOf(T o);
  
  /**
   * Gibt einen Iterator ueber die Elemente in dieser Liste in richtiger Reihenfolge
   * zurueck.
   *
   * @return einen Iterator ueber die Elemente in dieser Liste in richtiger Reihenfolge
   */
  public Iterator<T> iterator();

}
